@@ -1,23 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2019 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package Tickit::Async;
 
 use strict;
 use warnings;
 use base qw( Tickit IO::Async::Notifier );
-Tickit->VERSION( '0.67' );
 IO::Async::Notifier->VERSION( '0.43' ); # Need support for being a nonprinciple mixin
 
-our $VERSION = '0.21_002';
+our $VERSION = '0.10';
 
-use IO::Async::Loop 0.47; # ->run and ->stop methods
+use IO::Async::Loop;
+use IO::Async::Signal;
 use IO::Async::Stream;
-
-require XSLoader;
-XSLoader::load( __PACKAGE__, $VERSION );
+use Term::TermKey::Async;
 
 =head1 NAME
 
@@ -45,14 +43,11 @@ C<Tickit::Async> - use C<Tickit> with C<IO::Async>
 This class allows a L<Tickit> user interface to run alongside other
 L<IO::Async>-driven code, using C<IO::Async> as a source of IO events.
 
-As a shortcut convenience, a containing L<IO::Async::Loop> will be constructed
-using the default magic constructor the first time it is needed, if the object
-is not already a member of a loop. This will allow a C<Tickit::Async> object
-to be used without being aware it is not a simple C<Tickit> object.
-
-To avoid accidentally creating multiple loops, callers should be careful to
-C<add> the C<Tickit::Async> object to the main application's loop if one
-already exists as soon as possible after construction.
+As a shortcut convenience, if the C<run> method is invoked and the object is
+not yet a member of an L<IO::Async::Loop>, then a new one will be constructed
+and the C<Tickit::Async> object added to it. This will allow a
+C<Tickit::Async> object to be used without being aware it is not a simple
+C<Tickit> object.
 
 =cut
 
@@ -61,17 +56,26 @@ sub new
    my $class = shift;
    my $self = $class->Tickit::new( @_ );
 
+   $self->add_child( IO::Async::Signal->new( 
+      name => "WINCH",
+      on_receipt => $self->_capture_weakself( "_SIGWINCH" ),
+   ) );
+
    return $self;
 }
 
-sub get_loop
+sub _make_termkey
 {
    my $self = shift;
-   return $self->SUPER::get_loop || do {
-      my $newloop = IO::Async::Loop->new;
-      $newloop->add( $self );
-      $newloop;
-   };
+   my ( $in ) = @_;
+
+   my $tka = Term::TermKey::Async->new(
+      term => $in,
+      on_key => $self->_capture_weakself( "_KEY" ),
+   );
+   $self->add_child( $tka );
+
+   return $tka;
 }
 
 sub _make_writer
@@ -89,12 +93,60 @@ sub _make_writer
    return $writer;
 }
 
-sub _make_tickit
+sub _add_to_loop
 {
    my $self = shift;
-   my ( $term ) = @_;
 
-   return Tickit::Async::_new_tickit( $term, $self->get_loop );
+   $self->SUPER::_add_to_loop( @_ );
+
+   if( $self->{todo_queue} ) {
+      $self->get_loop->later( sub { $self->_flush_later } );
+   }
+}
+
+sub later
+{
+   my $self = shift;
+   my ( $code ) = @_;
+
+   if( my $loop = $self->get_loop ) {
+      $loop->later( $code );
+   }
+   else {
+      push @{ $self->{todo_queue} }, $code;
+   }
+}
+
+sub _STOP
+{
+   my $self = shift;
+   $self->get_loop->loop_stop;
+}
+
+sub run
+{
+   my $self = shift;
+
+   my $loop = $self->get_loop || do {
+      my $newloop = IO::Async::Loop->new;
+      $newloop->add( $self );
+      $newloop;
+   };
+
+   $self->start;
+
+   my $old_DIE = $SIG{__DIE__};
+   local $SIG{__DIE__} = sub {
+      local $SIG{__DIE__} = $old_DIE;
+
+      die @_ if $^S;
+
+      $self->stop;
+      die @_;
+   };
+
+   $loop->loop_forever;
+   $self->stop;
 }
 
 =head1 AUTHOR
